@@ -306,26 +306,69 @@ sub upgrade_add_file_to_queue_arg {
 
 # return 1 on success.  die otherwise.
 sub enqueue_fids_to_delete {
+    # My kingdom for a real INSERT IGNORE implementation!
     my ($self, @fidids) = @_;
     my $sql = "INSERT INTO file_to_delete (fid) VALUES (?)";
-    my $savepoint_name = "savepoint_enqueue_fids_to_delete";
 
-    $self->dbh->begin_work;
     foreach my $fidid (@fidids) {
-        $self->dbh->do('SAVEPOINT '.$savepoint_name);
+        $self->dbh->begin_work;
         $self->condthrow;
         eval {
             $self->dbh->do($sql, undef, $fidid);
         };
         if ($@ || $self->dbh->err) {
             if ($self->was_duplicate_error) {
-                $self->dbh->do('ROLLBACK TO '.$savepoint_name);
+                # Do nothing
+            } else {
+                $self->condthrow;
             }
-            $self->condthrow;
         }
+        $self->dbh->commit;
+    }
+
+}
+
+sub enqueue_fids_to_delete2 {
+    # My kingdom for a real REPLACE implementation!
+    my ($self, @fidids) = @_; 
+    my $tbl = 'file_to_delete2';
+    my $sql1 = sprintf "INSERT INTO %s (fid, nexttry) VALUES (?,%s)", $tbl, $self->unix_timestamp;
+    my @dup_fids;
+    
+    foreach my $fidid (@fidids) {
+        $self->dbh->begin_work;
+        $self->condthrow;
+        eval {
+            $self->dbh->do($sql1, undef, $fidid);
+        };
+        if ($@ || $self->dbh->err) {
+            if ($self->was_duplicate_error) {
+                push @dup_fids, $fidid;
+            } else {
+                $self->condthrow;
+            }
+        }
+        $self->dbh->commit;
+    }
+
+    my $sql2 = sprintf 'UPDATE %s SET nexttry = %s WHERE fid IN (?)', $tbl, $self->unix_timestamp;
+    
+    foreach my $fidid (@dup_fids) {
+        $self->dbh->begin_work;
+        $self->condthrow;
+        eval {
+            $self->dbh->do($sql2, undef, $fidid);
+        };
+        if ($@ || $self->dbh->err) {
+            if ($self->was_duplicate_error) {
+                # Ignore, no need of it
+            } else {
+                $self->condthrow;
+            }
+        }
+        $self->dbh->commit;
     }
 
-    $self->dbh->commit;
 }
 
 # --------------------------------------------------------------------------
